@@ -14,10 +14,13 @@ This module provides some extensions to matplotlib.
 """
 from __future__ import annotations
 
+import textwrap
+
 import deprecation
 import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
+import yaml
 from numpy.typing import ArrayLike
 from scipy.optimize import curve_fit
 
@@ -80,89 +83,74 @@ def copy_style():
 
     styles = {}
 
-    styles[
-        "goose.mplstyle"
-    ] = """
-figure.figsize       : 8,6
-font.weight          : normal
-font.size            : 16
-axes.labelsize       : medium
-axes.titlesize       : medium
-xtick.labelsize      : small
-ytick.labelsize      : small
-xtick.top            : True
-ytick.right          : True
-axes.facecolor       : none
-axes.prop_cycle      : cycler('color',['k', 'r', 'g', 'b', 'y', 'c', 'm'])
-legend.fontsize      : medium
-legend.fancybox      : true
-legend.columnspacing : 1.0
-legend.handletextpad : 0.2
-lines.linewidth      : 2
-image.cmap           : afmhot
-image.interpolation  : nearest
-image.origin         : lower
-savefig.facecolor    : none
-figure.autolayout    : True
-errorbar.capsize     : 2
-"""
+    styles["goose.mplstyle"] = {
+        "figure.figsize": "8, 6",
+        "font.weight": "normal",
+        "font.size": 16,
+        "axes.labelsize": "medium",
+        "axes.titlesize": "medium",
+        "xtick.labelsize": "small",
+        "ytick.labelsize": "small",
+        "xtick.top": True,
+        "ytick.right": True,
+        "axes.facecolor": "none",
+        "axes.prop_cycle": 'cycler("color",["k", "r", "g", "b", "y", "c", "m"])',
+        "legend.fontsize": "medium",
+        "legend.fancybox": True,
+        "legend.columnspacing": 1.0,
+        "legend.handletextpad": 0.2,
+        "lines.linewidth": 2,
+        "image.cmap": "afmhot",
+        "image.interpolation": "nearest",
+        "image.origin": "lower",
+        "savefig.facecolor": "none",
+        "errorbar.capsize": 2,
+    }
 
-    styles[
-        "goose-tick-in.mplstyle"
-    ] = """
-xtick.direction      : in
-ytick.direction      : in
-"""
+    styles["goose-autolayout.mplstyle"] = {
+        "figure.autolayout": True,
+    }
 
-    styles[
-        "goose-tick-lower.mplstyle"
-    ] = """
-xtick.top            : False
-ytick.right          : False
-axes.spines.top      : False
-axes.spines.right    : False
-"""
+    styles["goose-huge.mplstyle"] = {
+        "font.size": 20,
+    }
 
-    if system_has_latex() and find_latex_font_serif() is not None:
+    styles["goose-tick-in.mplstyle"] = {
+        "xtick.direction": "in",
+        "ytick.direction": "in",
+    }
 
-        styles[
-            "goose-latex.mplstyle"
-        ] = r"""
-font.family          : serif
-font.serif           : {serif:s}
-font.weight          : bold
-font.size            : 18
-text.usetex          : true
-text.latex.preamble  : \usepackage{{amsmath, amsfonts, amssymb, bm}}
-""".format(
-            serif=find_latex_font_serif()
-        )
+    styles["goose-tick-lower.mplstyle"] = {
+        "xtick.top": False,
+        "ytick.right": False,
+        "axes.spines.top": False,
+        "axes.spines.right": False,
+    }
 
-    elif system_has_latex():
+    styles["goose-latex.mplstyle"] = {
+        "font.family": "serif",
+        "font.serif": find_latex_font_serif(),
+        "font.weight": "bold",
+        "font.size": 18,
+        "text.usetex": True,
+        "text.latex.preamble": r"\usepackage{{amsmath, amsfonts, amssymb, bm}}",
+    }
 
-        styles[
-            "goose-latex.mplstyle"
-        ] = r"""
-font.family          : serif
-font.weight          : bold
-font.size            : 18
-text.usetex          : true
-text.latex.preamble  : \usepackage{{amsmath, amsfonts, amssymb, bm}}
-"""
-
-    else:
+    if not system_has_latex():
 
         import warnings
 
-        message = """LaTeX is not installed.
-To use LaTeX with 'goose-latex':
-1) Install LaTeX.
-2) Rerun 'GooseMPL.copy_style()'
-Until that time 'goose-latex' will be an empty style."""
+        message = textwrap.dedent(
+            """LaTeX is not installed.
+            To use LaTeX with "goose-latex":
+            1) Install LaTeX.
+            2) Rerun "GooseMPL.copy_style()"
+            Until that time "goose-latex" will be an empty style."""
+        )
 
         warnings.warn(message, Warning)
 
-        styles["goose-latex.mplstyle"] = ""
+        styles["goose-latex.mplstyle"] = {}
 
     # write style definitions
     # -----------------------
@@ -176,7 +164,8 @@ Until that time 'goose-latex' will be an empty style."""
 
     # write all styles
     for fname, style in styles.items():
-        open(os.path.join(dirname, fname), "w").write(style)
+        with open(os.path.join(dirname, fname), "w") as file:
+            yaml.dump(style, file)
 
 
 def latex_float(number, fmt="{0:.2g}"):
@@ -212,6 +201,200 @@ def latex_float(number, fmt="{0:.2g}"):
     return float_str
 
 
+def log_ticks(
+    lim: tuple(int, int) = None,
+    keep: list = None,
+    base: int | float = int(10),
+    axis: plt.Axes = None,
+    direction: str = "x",
+    minor: bool = False,
+) -> (list, list):
+    """
+    Get ticks and tick-labels between two bounds. For example::
+
+        >>> ticks, labels = log_ticks((0, 3))
+        >>> print(ticks)
+        [1, 10, 100, 1000]
+        >>> print(labels)
+        ['$10^{0}$', '$10^{1}$', '$10^{2}$', '$10^{3}$']
+
+    :param lim: Lower- and upper-bound exponent. Default: read from ``axis`` or ``plt.gca()``,
+    :param keep: Convert labels to empty strings.
+    :param base: The base of the exponents.
+    :param axis: Apply ticks/labels to an axis.
+    :param direction: "x" or "y".
+    :param minor: Use minor ticks: minor ticks are placed without labels.
+    :return: ticks, labels
+    """
+
+    direction = direction.lower()
+    output_only = axis is None
+    axis = axis if axis else plt.gca()
+
+    if direction == "x":
+        xdir = True
+    elif direction == "y":
+        xdir = False
+    else:
+        raise OSError("Unknown direction")
+
+    if lim is None:
+        if xdir:
+            lim = axis.get_xlim()
+        else:
+            lim = axis.get_ylim()
+        lim = [np.log(i) / np.log(base) for i in lim]
+        lim[0] = int(np.ceil(lim[0] - 0.1))
+        lim[1] = int(np.floor(lim[1] + 0.1))
+
+    exp_lower, exp_upper = lim
+    ticks = np.logspace(exp_lower, exp_upper, exp_upper - exp_lower + 1, base=base)
+    labels = [fr"${base}^{{{np.log10(i):.0f}}}$" for i in ticks]
+
+    if keep is not None:
+        keep = np.array(keep)
+        keep[keep < 0] = len(labels) + keep[keep < 0]
+        for i in np.setdiff1d(np.arange(len(labels)), keep):
+            labels[i] = ""
+
+    if output_only:
+        return ticks, labels
+
+    if xdir:
+        axis.set_xticks(ticks)
+        axis.set_xticklabels(labels)
+        if not minor:
+            axis.set_xticks([], minor=True)
+    else:
+        axis.set_yticks(ticks)
+        axis.set_yticklabels(labels)
+        if not minor:
+            axis.set_yticks([], minor=True)
+
+    if minor:
+        assert base == 10
+        log_minorticks(
+            lim=None,
+            keep=[],
+            axis=axis,
+            direction=direction,
+        )
+
+    return ticks, labels
+
+
+def log_xticks(*args, **kwargs):
+    """
+    See :py:func:`log_ticks`.
+    """
+    kwargs.setdefault("direction", "x")
+    return log_ticks(*args, **kwargs)
+
+
+def log_yticks(*args, **kwargs):
+    """
+    See :py:func:`log_ticks`.
+    """
+    kwargs.setdefault("direction", "y")
+    return log_ticks(*args, **kwargs)
+
+
+def log_minorticks(
+    lim: tuple(float, float) = None,
+    keep: list = None,
+    axis: plt.Axes = None,
+    direction: str = "x",
+    integer: bool = False,
+) -> (list, list):
+    """
+    Get minor ticks and tick-labels between two bounds.
+
+    :param lim: Lower- and upper-bound. Default: read from ``axis`` or ``plt.gca()``,
+    :param keep: Convert labels to empty strings, except at given indices.
+    :param axis: Apply ticks/labels to an axis.
+    :param direction: "x" or "y".
+    :param integer: Show numbers as integer (if possible).
+    :return: ticks, labels
+    """
+
+    direction = direction.lower()
+    output_only = axis is None
+    axis = axis if axis else plt.gca()
+
+    if direction == "x":
+        xdir = True
+    elif direction == "y":
+        xdir = False
+    else:
+        raise OSError("Unknown direction")
+
+    if lim is None:
+        if xdir:
+            lim = axis.get_xlim()
+        else:
+            lim = axis.get_ylim()
+
+    exp_lower = int(np.floor(np.log10(lim[0])))
+    exp_upper = int(np.ceil(np.log10(lim[1])))
+
+    ticks = []
+    labels = []
+
+    for i in range(exp_lower, exp_upper):
+
+        t = list((10 ** i) * np.arange(2, 10, dtype=float))
+
+        if i < 0:
+            fmt = "{0:.0%df}" % np.abs(i)
+        elif integer:
+            fmt = "{0:.0f}"
+        else:
+            fmt = "{0:.01f}"
+
+        ticks += t
+        labels += [fmt.format(i) for i in t]
+
+    ticks = np.array(ticks)
+    labels = np.array(labels)
+    i = np.logical_and(ticks >= lim[0], ticks <= lim[1])
+    ticks = list(ticks[i])
+    labels = [str(i) for i in labels[i]]
+
+    if keep is not None:
+        keep = np.array(keep)
+        keep[keep < 0] = len(labels) + keep[keep < 0]
+        for i in np.setdiff1d(np.arange(len(labels)), keep):
+            labels[i] = ""
+
+    if output_only:
+        return ticks, labels
+
+    if xdir:
+        axis.set_xticks(ticks, minor=True)
+        axis.set_xticklabels(labels, minor=True)
+    else:
+        axis.set_yticks(ticks, minor=True)
+        axis.set_yticklabels(labels, minor=True)
+
+    return ticks, labels
+
+
+def log_minorxticks(*args, **kwargs):
+    """
+    See :py:func:`log_minorticks`.
+    """
+    kwargs.setdefault("direction", "x")
+    return log_minorticks(*args, **kwargs)
+
+
+def log_minoryticks(*args, **kwargs):
+    """
+    See :py:func:`log_minorticks`.
+    """
+    kwargs.setdefault("direction", "y")
+    return log_minorticks(*args, **kwargs)
+
+
 def minmax(a):
     r"""
     Return [np.min(a), np.max(a)]
@@ -1014,50 +1197,20 @@ def grid_powerlaw(exp, insert=0, skip=0, end=-1, step=0, axis=None, **kwargs):
     return lines
 
 
-def fit_powerlaw(
-    xdata: ArrayLike,
-    ydata: ArrayLike,
-    prefactor: float = None,
-    exponent: float = None,
-    axis: plt.Axes = None,
-    fmt: str = None,
-    **kwargs,
-):
-    r"""
-    Fit a powerlaw by converting the data to their logarithm and fitting a straight line.
-    This function does not support more customised operation like fitting an offset,
-    but custom code can be easily written by copy/pasting from here.
-
-    Warning: If this function is used to plot the fit, beware that the fit is plotted using just
-    two data-points if the axis is set to log-log scale
-    (as the fit will be a straight line on that scale).
-
-    :param xdata: Data points along the x-axis.
-    :param ydata: Data points along the y-axis.
-    :param prefactor: Prefactor (fitted if not specified).
-    :param exponent: Exponent (fitted if not specified).
-    :param axis: Axis to plot along (not plotted if not specified).
-    :param fmt: Format for the label (if plotting). E.g. ``r"${{{0:.3f}}} x^{{{1:.2f}}}$"``
-    :param kwargs: Other plot options
-
-    :return:
-        ``prefactor, exponent[, h]``
-        The (fitted) prefector and exponent, and optionally the handle (if plotting)
-    """
-
-    i = np.logical_and(xdata > 0, ydata > 0)
-    logx = np.log(xdata[i])
-    logy = np.log(ydata[i])
-    i = np.logical_or(np.isnan(logx), np.isnan(logy))
-    logy = logy[~i]
-    logx = logx[~i]
+def _fit_powerlaw(
+    logx: ArrayLike,
+    logy: ArrayLike,
+    prefactor: float,
+    exponent: float,
+    **fit_opts,
+) -> (float, float):
 
     if prefactor is None and exponent is None:
 
         def f(logx, log_prefactor, exponent):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy)
+        param, _ = curve_fit(f, logx, logy, **fit_opts)
         prefactor = np.exp(param[0])
         exponent = param[1]
 
@@ -1066,7 +1219,7 @@ def fit_powerlaw(
         def f(logx, log_prefactor):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy)
+        param, _ = curve_fit(f, logx, logy, **fit_opts)
         prefactor = np.exp(param[0])
 
     elif exponent is None:
@@ -1076,9 +1229,64 @@ def fit_powerlaw(
         def f(logx, exponent):
             return log_prefactor + exponent * logx
 
-        param, _ = curve_fit(f, logx, logy)
+        param, _ = curve_fit(f, logx, logy, **fit_opts)
         exponent = param[0]
 
+    return prefactor, exponent
+
+
+def fit_powerlaw(
+    xdata: ArrayLike,
+    ydata: ArrayLike,
+    yerr: ArrayLike = None,
+    prefactor: float = None,
+    exponent: float = None,
+    axis: plt.Axes = None,
+    fmt: str = None,
+    **kwargs,
+):
+    r"""
+    Fit a powerlaw :math:`y = c x^b` by converting
+    both :math:`x` and :math:`y` to their logarithm and fitting a straight line.
+    This function does not support more customised operation like fitting an offset,
+    but custom code can be easily written by copy/pasting from here.
+
+    Warning: If this function is used to plot the fit, beware that the fit is plotted using just
+    two data-points if the axis is set to log-log scale
+    (as the fit will be a straight line on that scale).
+
+    :param xdata: Data points along the x-axis.
+    :param ydata: Data points along the y-axis.
+    :param yerr: Error-bar for ``ydata``.
+    :param prefactor: Prefactor (fitted if not specified).
+    :param exponent: Exponent (fitted if not specified).
+    :param axis: Axis to plot along (not plotted if not specified).
+    :param fmt: Format for the label (if plotting). E.g. ``r"${0:.3f} x^{{{1:.2f}}}$"``.
+    :param kwargs: Other plot options.
+
+    :return:
+        ``prefactor, exponent[, h]``
+        The (fitted) prefector and exponent, and optionally the handle (if plotting)
+    """
+
+    i = np.logical_and(xdata > 0, ydata > 0)
+    logx = np.log(xdata[i])
+    logy = np.log(ydata[i])
+
+    j = np.logical_or(np.isnan(logx), np.isnan(logy))
+    logx = logx[~j]
+    logy = logy[~j]
+
+    fit_opts = {}
+
+    if yerr is not None:
+        logyerr = np.log(yerr[i])
+        logyerr = logyerr[~j]
+        fit_opts["sigma"] = logyerr
+        fit_opts["absolute_sigma"] = True
+
+    prefactor, exponent = _fit_powerlaw(logx, logy, prefactor, exponent, **fit_opts)
+
     if axis is None:
         return (prefactor, exponent)
 
@@ -1098,6 +1306,77 @@ def fit_powerlaw(
     return (prefactor, exponent, h)
 
 
+def fit_exp(
+    xdata: ArrayLike,
+    ydata: ArrayLike,
+    yerr: ArrayLike = None,
+    prefactor: float = None,
+    exponent: float = None,
+    axis: plt.Axes = None,
+    fmt: str = None,
+    **kwargs,
+):
+    r"""
+    Fit an exponential :math:`y = c \exp(b x)` by converting
+    :math:`y` to its logarithm and fitting a straight line.
+    This function does not support more customised operation like fitting an offset,
+    but custom code can be easily written by copy/pasting from here.
+
+    Warning: If this function is used to plot the fit, beware that the fit is plotted using just
+    two data-points if the axis is set to semilogy-scale
+    (as the fit will be a straight line on that scale).
+
+    :param xdata: Data points along the x-axis.
+    :param ydata: Data points along the y-axis.
+    :param yerr: Error-bar for ``ydata``.
+    :param prefactor: Prefactor (fitted if not specified).
+    :param exponent: Exponent (fitted if not specified).
+    :param axis: Axis to plot along (not plotted if not specified).
+    :param fmt: Format for the label (if plotting). E.g. ``r"${0:.3f} \exp ({1:.2f} x)$"``.
+    :param kwargs: Other plot options.
+
+    :return:
+        ``prefactor, exponent[, h]``
+        The (fitted) prefector and exponent, and optionally the handle (if plotting)
+    """
+
+    i = ydata > 0
+    x = xdata[i]
+    logy = np.log(ydata[i])
+
+    j = np.isnan(logy)
+    logy = logy[~j]
+    x = x[~j]
+
+    fit_opts = {}
+
+    if yerr is not None:
+        logyerr = np.log(yerr[i])
+        logyerr = logyerr[~j]
+        fit_opts["sigma"] = logyerr
+        fit_opts["absolute_sigma"] = True
+
+    prefactor, exponent = _fit_powerlaw(x, logy, prefactor, exponent, **fit_opts)
+
+    if axis is None:
+        return (prefactor, exponent)
+
+    xp = np.array([np.min(x), np.max(x)])
+
+    if axis.get_yscale() != "log":
+        xp = np.linspace(xp[0], xp[-1], 1000)
+
+    yp = prefactor * np.exp(exponent * xp)
+
+    if fmt:
+        assert "label" not in kwargs
+        kwargs["label"] = fmt.format(prefactor, exponent)
+
+    h = axis.plot(xp, yp, **kwargs)
+
+    return (prefactor, exponent, h)
+
+
 def random_from_cdf(shape, P, x, linspace=False, shuffle=True):
     r"""
     Generate a random number based on a discrete cumulative probability density function.
